{
 Copyright (C) 2013 Dimitar Paperov

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA.

}
unit mainform;

{$mode objfpc}{$H+}

interface
{$region ' uses '}
uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ActnList {$IFNDEF WINDOWS}, types{$ENDIF};
{$endregion}

type
  { TForm1 }
  TForm1 = class(TForm)
    acLoadLib: TAction;
    acFreeLib: TAction;
    acNewAppId: TAction;
    acEthInit: TAction;
    acCOMinit: TAction;
    acGetAngle: TAction;
    acGetAngles: TAction;
    acGetConfig: TAction;
    acGetRotorConfig: TAction;
    acLogon: TAction;
    acLogout: TAction;
    acSetRotorConfig: TAction;
    acSetAngle: TAction;
    acSetAngles: TAction;
    acSetConfig: TAction;
    al: TActionList;
    btnLoad: TButton;
    btnFree: TButton;
    btn_angle1: TButton;
    btn_angle2: TButton;
    btn_angle3: TButton;
    btn_angle4: TButton;
    btn_angle5: TButton;
    btn_angle6: TButton;
    btn_angle7: TButton;
    btn_angle8: TButton;
    btn_angle9: TButton;
    btn_md_eth_init: TButton;
    btn_md_com_init: TButton;
    btn_md_new_app_id: TButton;
    btn_angle: TButton;
    lbAppId: TLabel;
    procedure acCOMinitExecute(Sender: TObject);
    procedure acEthInitExecute(Sender: TObject);
    procedure acFreeLibExecute(Sender: TObject);
    procedure acGetAngleExecute(Sender: TObject);
    procedure acGetAnglesExecute(Sender: TObject);
    procedure acGetConfigExecute(Sender: TObject);
    procedure acGetRotorConfigExecute(Sender: TObject);
    procedure acLoadLibExecute(Sender: TObject);
    procedure acLogonExecute(Sender: TObject);
    procedure acLogoutExecute(Sender: TObject);
    procedure acNewAppIdExecute(Sender: TObject);
    procedure acSetAngleExecute(Sender: TObject);
    procedure acSetAnglesExecute(Sender: TObject);
    procedure acSetConfigExecute(Sender: TObject);
    procedure acSetRotorConfigExecute(Sender: TObject);
    procedure alUpdate(AAction: TBasicAction; var Handled: Boolean);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
    fLibLoaded: boolean;
    fCommChannelOpened: boolean;
    fLogon: boolean;
    function AppIdToString(p: pointer): string;
  public
    { public declarations }
  end;

var
  Form1: TForm1;

implementation

uses mdlib, angle_form, motor_config_form, motor_params_form, mdtypes;

var app_id: TMD_APP_ID = (hiPart:0; loPart:0);

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  lbAppId.Caption := 'App ID: ' + AppIdToString(@app_id);
  fLibLoaded := false;
  fCommChannelOpened := false;
  fLogon := false;
end;

function TForm1.AppIdToString(p: pointer): string;
var id: PMD_APP_ID;
begin
  id := p;
  if (id^.loPart=0) and (id^.hiPart=0) then
    Result := 'none'
  else
    Result := Format('%.8X,%.8X',[id^.hiPart, id^.loPart]);
end;
{$HINTS OFF}
procedure TForm1.alUpdate(AAction: TBasicAction; var Handled: Boolean);
begin
  acCOMinit.Enabled := fLibLoaded;
  acEthInit.Enabled := fLibLoaded;
  acFreeLib.Enabled := fLibLoaded;
  acNewAppId.Enabled := fLibLoaded;
  acLoadLib.Enabled := not fLibLoaded;
  acGetAngle.Enabled := fCommChannelOpened;
  acGetAngles.Enabled := fCommChannelOpened;
  acGetConfig.Enabled := fCommChannelOpened;
  acGetRotorConfig.Enabled := fCommChannelOpened;
  acSetAngle.Enabled := fLogon;
  acSetAngles.Enabled := fLogon;
  acSetRotorConfig.Enabled := fLogon;
  acSetConfig.Enabled := fLogon;
  acLogon.Enabled := fCommChannelOpened and (app_id.hiPart<>0) and (app_id.loPart<>0);
  acLogout.Enabled := fLogon  and fCommChannelOpened and (app_id.hiPart<>0) and (app_id.loPart<>0);
  Handled := true;
end;

{$HINTS ON}

procedure TForm1.acLoadLibExecute(Sender: TObject);
begin
  if spidcomm_load('') then begin
    fLibLoaded := true;
    ShowMessage('DLL loaded');
  end else begin
    fLibLoaded := false;
    fCommChannelOpened := false;
    ShowMessage('DLL not loaded');
  end;
end;

procedure TForm1.acLogonExecute(Sender: TObject);
var p: TMD_LOGON_PARAMS;
begin
  p.dev.group := 1;
  p.dev.number := 1;
  p.app := app_id;
  fLogon := false;
  case md_logon(@p) of
    MD_NO_ERROR: begin
      fLogon := true;
      ShowMessage('Ok');
    end;
    MD_DEVICE_BUSY: MessageDlg('Test','Device is used by another user.', mtError, [mbOK], 0);
    MD_BAD_PARAMETERS: MessageDlg('Test','Bad parameters.', mtError, [mbOK], 0);
  end;
end;

procedure TForm1.acLogoutExecute(Sender: TObject);
var p: TMD_LOGON_PARAMS;
begin
  p.dev.group := 1;
  p.dev.number := 1;
  p.app := app_id;
  case md_logout(@p) of
    MD_NO_ERROR: begin
      fLogon := false;
      ShowMessage('Ok');
    end;
    MD_DEVICE_BUSY: MessageDlg('Test','Device is used by another user.', mtError, [mbOK], 0);
    MD_BAD_PARAMETERS: MessageDlg('Test','Bad parameters.', mtError, [mbOK], 0);
  end;
end;

procedure TForm1.acNewAppIdExecute(Sender: TObject);
var id: TMD_APP_ID;
begin
  if md_new_app_id(@id)=MD_NO_ERROR then begin
    if (app_id.hiPart <> id.hiPart) and (app_id.loPart <> id.loPart) then begin
      app_id := id;
      fLogon := false;
    end;
  end;
  lbAppId.Caption := 'App ID: ' + AppIdToString(@app_id);
end;

procedure TForm1.acSetAngleExecute(Sender: TObject);
var p: TMD_ROTOR_ANGLE_PARAMS;
begin
  if Form2.ShowMe(1) then begin
    if Form2.CheckBox1.Checked then
      p.calibration:=1
    else
      p.calibration:=0;
    p.angle:=StrToFloat(Form2.Edit1.Text);
    p.dev.group:=1;
    p.dev.number:=1;
    p.rotorIx:=0;
    case md_set_rotor_angle(@app_id, @p) of
      MD_NO_ERROR: ShowMessage('Ok');
      else
        ShowMessage('error');
    end;
  end;
end;

procedure TForm1.acSetAnglesExecute(Sender: TObject);
var p: TMD_ALL_ROTOR_ANGLES_PARAMS;
begin
  if Form2.ShowMe(2) then begin
    if Form2.CheckBox1.Checked then
      p.calibration:=1
    else
      p.calibration:=0;
    p.angles[0]:=StrToFloat(Form2.Edit1.Text);
    p.angles[1]:=StrToFloat(Form2.Edit2.Text);
    p.startMotorIx:=0;
    p.dev.group:=1;
    p.dev.number:=1;
    case md_set_all_rotor_angles(@app_id, @p) of
      MD_NO_ERROR: ShowMessage('Ok');
      else
        ShowMessage('error');
    end;
  end;
end;

procedure TForm1.acSetConfigExecute(Sender: TObject);
var p: TMD_CONFIG_PARAMS;
    c: TMD_CONFIG;
begin
  p.dev.number:=1;
  p.dev.group:=1;
  p.config := @c;
  if md_get_config(@p)=MD_NO_ERROR then begin
    Form3.ToInterface(c);
    if Form3.ShowMe(true) then begin
      Form3.FromInterface(c);
      case md_set_config(@app_id, @p) of
        MD_NO_ERROR: ShowMessage('Ok');
        else
          ShowMessage('Error!');
      end;
    end;
  end;
end;

procedure TForm1.acSetRotorConfigExecute(Sender: TObject);
var p: TMD_ROTOR_CONFIG_PARAMS;
    c: TMD_ROTOR_CONFIG;
    s: string;
begin
  s := '0';
  if InputQuery('Test','ROTOR number (from zero)', s) then begin
    p.rotorIx:=StrToIntDef(s, 0);
    p.dev.number:=1;
    p.dev.group:=1;
    p.config := @c;
    if md_get_rotor_config(@p) = MD_NO_ERROR then begin
      frmRotorParams.ToInterface(c);
      if frmRotorParams.ShowMe(false) then begin
        frmRotorParams.FromInterface(c);
        if md_set_rotor_config(@app_id, @p)=MD_NO_ERROR then
          ShowMessage('Ok')
        else
          ShowMessage('Error.');
      end;
    end;
  end;
end;

procedure TForm1.acFreeLibExecute(Sender: TObject);
begin
  if spidcomm_free then begin
    fLibLoaded := false;
    fCommChannelOpened := false;
    ShowMessage('DLL unloaded');
  end else begin
    ShowMessage('DLL not unloaded');
  end;
end;

procedure TForm1.acGetAngleExecute(Sender: TObject);
var s: string;
    p: TMD_ROTOR_ANGLE_PARAMS;
begin
  s := '0';
  if InputQuery('Test','ROTOR number (from zero)', s) then begin
    p.rotorIx:=StrToIntDef(s, 0);
    p.dev.number:=1;
    p.dev.group:=1;
    p.angle:=0;
    case md_get_rotor_angle(@p) of
      MD_NO_ERROR: ShowMessage(Format('ROTOR %s: %.1f',[s, p.angle]));
      MD_BAD_PARAMETERS: ShowMessage('Bad parameters.');
      else
        ShowMessage('Unexpected error');
    end;
  end;
end;

procedure TForm1.acGetAnglesExecute(Sender: TObject);
var p: TMD_ALL_ROTOR_ANGLES_PARAMS;
begin
  p.startMotorIx := 0;
  p.dev.number:=1;
  p.dev.group:=1;
  case md_get_all_rotor_angles(@p) of
    MD_NO_ERROR: ShowMessage(Format('ROTOR 0: %.1f, ROTOR 1: %.1f',[p.angles[0],p.angles[1]]));
    MD_BAD_PARAMETERS: ShowMessage('Bad parameters.');
    else
      ShowMessage('Unexpected error');
  end;
end;

procedure TForm1.acGetConfigExecute(Sender: TObject);
var p: TMD_CONFIG_PARAMS;
    c: TMD_CONFIG;
begin
  p.dev.number:=1;
  p.dev.group:=1;
  p.config := @c;
  case md_get_config(@p) of
    MD_NO_ERROR: begin
      Form3.ToInterface(c);
      Form3.ShowMe(false);
    end;
    MD_BAD_PARAMETERS: ShowMessage('Bad parameters.');
    else
      ShowMessage('Unexpected error');
  end;
end;

procedure TForm1.acGetRotorConfigExecute(Sender: TObject);
var p: TMD_ROTOR_CONFIG_PARAMS;
    c: TMD_ROTOR_CONFIG;
    s: string;
begin
  s := '0';
  if InputQuery('Test','ROTOR number (from zero)', s) then begin
    p.rotorIx:=StrToIntDef(s, 0);
    p.dev.number:=1;
    p.dev.group:=1;
    p.config := @c;
    case md_get_rotor_config(@p) of
      MD_NO_ERROR: begin
        frmRotorParams.Caption := 'CONFGI OF MOTOR ' + s;
        frmRotorParams.ToInterface(c);
        frmRotorParams.ShowMe(true);
      end;
      MD_BAD_PARAMETERS: ShowMessage('Bad parameters.');
      else
        ShowMessage('Unexpected error');
    end;
  end;
end;

procedure TForm1.acEthInitExecute(Sender: TObject);
var s: string;
begin
  s := '192.168.0.1';
  if InputQuery('Tets','Host',s) then begin
    fCommChannelOpened := false;
    case md_eth_init(PChar(s), 23) of
      MD_BAD_PARAMETERS: ShowMessage('Bad parameters.');
      MD_HOST_NOT_EXISTS: ShowMessage('Host not exists.');
      MD_NO_ERROR: begin
        fCommChannelOpened := true;
        ShowMessage('Success.');
      end
      else
        ShowMessage('Unexpected error');
    end;
  end;
end;

procedure TForm1.acCOMinitExecute(Sender: TObject);
var s: string;
    i: integer;
begin
  {$IFDEF WINDOWS}
  s := 'COM1';
  {$ELSE}
  s := 'ttyUSB0';
  {$ENDIF}
  if InputQuery('Test','COM port number', s) then begin
    if (s='COM1') or (s='COM2') then
      i := 115200
    else
      i := 460800;
    fCommChannelOpened := false;
    case md_com_init(PChar(s), i) of
      MD_BAD_PARAMETERS: ShowMessage('Bad parameters.');
      MD_COM_PORT_NOT_OPENED: ShowMessage('COM port not opened.');
      MD_NO_ERROR: begin
        fCommChannelOpened := true;
        ShowMessage('Success.');
      end
      else
        ShowMessage('Unexpected error');
    end;
  end;
end;

end.

