{ Copyright (C) 2013 Dimitar Paperov

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA.
}

unit motor_params_form;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls, Spin, mdTypes;

type

  { TfrmRotorParams }

  TfrmRotorParams = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Gear: TFloatSpinEdit;
    gAngles: TGroupBox;
    gStartRamp: TGroupBox;
    gStopRamp: TGroupBox;
    gOther: TGroupBox;
    Label1: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label2: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;
    Label27: TLabel;
    Label28: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    MaxAngle: TSpinEdit;
    MaxPower: TSpinEdit;
    StopAt: TSpinEdit;
    StopTime2: TSpinEdit;
    StopTime3: TSpinEdit;
    StopPower2: TSpinEdit;
    StopPower3: TSpinEdit;
    StartTime1: TSpinEdit;
    StartPower1: TSpinEdit;
    StartTime2: TSpinEdit;
    StartTime3: TSpinEdit;
    StartPower2: TSpinEdit;
    StartPower3: TSpinEdit;
    StopTime1: TSpinEdit;
    StopPower1: TSpinEdit;
    MinAngle: TSpinEdit;
    PulsesTimeout: TSpinEdit;
    gState: TRadioGroup;
    gType: TRadioGroup;
    gKing: TRadioGroup;
    gInput: TRadioGroup;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  private
    { private declarations }
    fReadOnly: boolean;
  public
    { public declarations }
    function ShowMe(aReadOnly: boolean): boolean;
    procedure ToInterface(var c: TMD_ROTOR_CONFIG);
    procedure FromInterface(var c: TMD_ROTOR_CONFIG);
  end;

var
  frmRotorParams: TfrmRotorParams;

implementation

{$R *.lfm}

{ TfrmRotorParams }

procedure TfrmRotorParams.Button1Click(Sender: TObject);
begin
  if fReadOnly then
    ModalResult:=mrCancel
  else
    ModalResult := mrOk;
end;

procedure TfrmRotorParams.Button2Click(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TfrmRotorParams.Button3Click(Sender: TObject);
begin
  gType.ItemIndex := 0;
  gInput.ItemIndex := 1;
  if gKing.ItemIndex = 0 then begin
    MinAngle.Value := -180;
    MaxAngle.Value := 540;
  end else begin
    MinAngle.Value := -15;
    MaxAngle.Value := 195;
  end;
  Gear.Value := 0.09375;
  PulsesTimeout.Value := 4;
  MaxPower.Value := 100;
  StopAt.Value := 5;
  StartPower1.Value := 0;
  StartPower2.Value := 0;
  StartPower3.Value := 0;
  StartTime1.Value := 0;
  StartTime2.Value := 0;
  StartTime3.Value := 0;
  StopPower1.Value := 0;
  StopPower2.Value := 0;
  StopPower3.Value := 0;
  StopTime1.Value := 0;
  StopTime2.Value := 0;
  StopTime3.Value := 0;
end;

function TfrmRotorParams.ShowMe(aReadOnly: boolean): boolean;
begin
  fReadOnly := aReadOnly;
  gState.Enabled:=false;  // always readonly
  gType.Enabled:=false; // always readonly
  gKing.Enabled:=false; // always readonly
  gInput.Enabled:=not aReadOnly;
  gAngles.Enabled:=not aReadOnly;
  gStartRamp.Enabled:=not aReadOnly;
  gStopRamp.Enabled:=not aReadOnly;
  gOther.Enabled:=not aReadOnly;
  if aReadOnly then begin
    Button2.Visible:=false;
    Button3.Visible:=false;
  end else begin
    Button2.Visible:=true;
    Button3.Visible:=true;
  end;
  Result := ShowModal = mrOk;
end;

procedure TfrmRotorParams.ToInterface(var c: TMD_ROTOR_CONFIG);
begin
  gState.ItemIndex    := c.state;
  gType.ItemIndex     := c.m_type;
  gKing.ItemIndex     := c.kind;
  gInput.ItemIndex    := c.input;
  MinAngle.Value      := Round(c.min_angle);
  MaxAngle.Value      := Round(c.max_angle);
  StartTime1.Value    := c.start_time[0];
  StartTime2.Value    := c.start_time[1];
  StartTime3.Value    := c.start_time[2];
  StartPower1.Value   := c.start_power[0];
  StartPower2.Value   := c.start_power[1];
  StartPower3.Value   := c.start_power[2];
  StopTime1.Value     := c.stop_time[0];
  StopTime2.Value     := c.stop_time[1];
  StopTime3.Value     := c.stop_time[2];
  StopPower1.Value    := c.stop_power[0];
  StopPower2.Value    := c.stop_power[1];
  StopPower3.Value    := c.stop_power[2];
  MaxPower.Value      := c.max_power;
  StopAt.Value        := Round(c.stop_at);
  PulsesTimeout.Value := c.puls_timeout;
  Gear.Value          := c.gear;
end;

procedure TfrmRotorParams.FromInterface(var c: TMD_ROTOR_CONFIG);
begin
  c.state          := gState.ItemIndex;
  c.m_type         := gType.ItemIndex;
  c.kind           := gKing.ItemIndex;
  c.input          := gInput.ItemIndex;
  c.min_angle      := MinAngle.Value;
  c.max_angle      := MaxAngle.Value;
  c.start_time[0]  := StartTime1.Value;
  c.start_time[1]  := StartTime2.Value;
  c.start_time[2]  := StartTime3.Value;
  c.start_power[0] := StartPower1.Value;
  c.start_power[1] := StartPower2.Value;
  c.start_power[2] := StartPower3.Value;
  c.stop_time[0]   := StopTime1.Value;
  c.stop_time[1]   := StopTime2.Value;
  c.stop_time[2]   := StopTime3.Value;
  c.stop_power[0]  := StopPower1.Value;
  c.stop_power[1]  := StopPower2.Value;
  c.stop_power[2]  := StopPower3.Value;
  c.max_power      := MaxPower.Value;
  c.stop_at        := StopAt.Value;
  c.puls_timeout   := PulsesTimeout.Value;
  c.gear           := Gear.Value;

end;

end.

